@@ -1,7 +1,7 @@
 import logging
 import os
 
-from django.core.files.uploadedfile import UploadedFile, InMemoryUploadedFile
+from django.core.files.uploadedfile import UploadedFile
 from rest_framework import status
 from rest_framework.exceptions import ParseError, MethodNotAllowed
 from rest_framework.response import Response
@@ -9,7 +9,7 @@ from rest_framework.response import Response
 from django_drf_filepond.models import TemporaryUpload, storage,\
     TemporaryUploadChunked
 from io import BytesIO, StringIO
-from django_drf_filepond.utils import _get_user
+from django_drf_filepond.utils import DrfFilepondChunkedUploadedFile, _get_user
 from six import text_type, binary_type
 
 # There's no built in FileNotFoundError in Python 2
@@ -61,7 +61,7 @@ class FilepondFileUploader(object):
             upload_field_name = request.data['fp_upload_field']
 
         if upload_field_name not in request.data:
-            raise ParseError('Invalid request data has been provided.')
+            raise ParseError('Could not find upload_field_name in request data.')
 
         # The content of the upload field is a django.http.QueryDict.
         # The dict may have multiple values for a given field name.
@@ -174,6 +174,9 @@ class FilepondChunkedFileUploader(FilepondFileUploader):
                 return Response('Invalid ID for handling upload.',
                                 status=status.HTTP_500_INTERNAL_SERVER_ERROR)
             return self._handle_new_chunk_upload(request, upload_id, file_id)
+        else:
+            return Response('Invalid method.',
+                            status=status.HTTP_405_METHOD_NOT_ALLOWED)
 
     def _handle_new_chunk_upload(self, request, upload_id, file_id):
         LOG.debug('Processing a new chunked upload request...')
@@ -232,6 +235,11 @@ class FilepondChunkedFileUploader(FilepondFileUploader):
         # the data couldn't be handled by the configured parser.
         file_data = request.data
 
+        # Get the required header information to handle the new data
+        uoffset = request.META.get('HTTP_UPLOAD_OFFSET', None)
+        ulength = request.META.get('HTTP_UPLOAD_LENGTH', None)
+        uname = request.META.get('HTTP_UPLOAD_NAME', None)
+
         if (not chunk_id) or (chunk_id == ''):
             return Response('A required chunk parameter is missing.',
                             status=status.HTTP_400_BAD_REQUEST)
@@ -240,6 +248,19 @@ class FilepondChunkedFileUploader(FilepondFileUploader):
             fd = BytesIO(file_data)
         elif isinstance(file_data, text_type):
             fd = StringIO(file_data)
+        # If file_data is an invalid type and this is not iterable the
+        # next check fails so need to support this case.
+        elif hasattr(file_data, '__iter__') and len(file_data) == 0:
+            # This may be a final empty request in relation to a completed
+            # upload - see issue #72
+            # Check the headers and accept the request if the offset is the
+            # same as the length (i.e. all file data has already been sent).
+            if uoffset and ulength and (uoffset == ulength):
+                return Response(chunk_id, status=status.HTTP_200_OK,
+                                content_type='text/plain')
+            else:
+                return Response('Upload data type not recognised.',
+                                status=status.HTTP_400_BAD_REQUEST)
         else:
             return Response('Upload data type not recognised.',
                             status=status.HTTP_400_BAD_REQUEST)
@@ -251,11 +272,7 @@ class FilepondChunkedFileUploader(FilepondFileUploader):
             return Response('Invalid chunk upload request data',
                             status=status.HTTP_400_BAD_REQUEST)
 
-        # Get the required header information to handle the new data
-        uoffset = request.META.get('HTTP_UPLOAD_OFFSET', None)
-        ulength = request.META.get('HTTP_UPLOAD_LENGTH', None)
-        uname = request.META.get('HTTP_UPLOAD_NAME', None)
-
+        # Now check that the required headers were set
         if (uoffset is None) or (ulength is None) or (uname is None):
             return Response('Chunk upload is missing required metadata',
                             status=status.HTTP_400_BAD_REQUEST)
@@ -329,29 +346,35 @@ class FilepondChunkedFileUploader(FilepondFileUploader):
 
         # Load each of the file parts into a BytesIO object and store them
         # via a TemporaryUpload object.
-        chunk_dir = os.path.join(storage.base_location, tuc.upload_dir)
-        file_data = BytesIO()
-        for i in range(1, tuc.last_chunk+1):
-            chunk_file = os.path.join(chunk_dir, '%s_%s' % (tuc.file_id, i))
-            if not os.path.exists(chunk_file):
-                raise FileNotFoundError('Chunk file not found for chunk <%s>'
-                                        % (i))
+        # chunk_dir = os.path.join(storage.base_location, tuc.upload_dir)
+        # file_data = BytesIO()
+        # for i in range(1, tuc.last_chunk+1):
+        #     chunk_file = os.path.join(chunk_dir, '%s_%s' % (tuc.file_id, i))
+        #     if not os.path.exists(chunk_file):
+        #         raise FileNotFoundError('Chunk file not found for chunk <%s>'
+        #                                 % (i))
 
-            with open(chunk_file, 'rb') as cf:
-                file_data.write(cf.read())
+        #     with open(chunk_file, 'rb') as cf:
+        #         file_data.write(cf.read())
+
+        # # Prepare an InMemoryUploadedFile object so that the data can be
+        # # successfully saved via the FileField in the TemporaryUpload object
+        # memfile = InMemoryUploadedFile(file_data, None, tuc.file_id,
+        #                                'application/octet-stream',
+        #                                tuc.total_size, None)
+
+        chunked_file = DrfFilepondChunkedUploadedFile(
+            tuc, 'application/octet-stream')
+        chunked_file.open('rb')
 
-        # Prepare an InMemoryUploadedFile object so that the data can be
-        # successfully saved via the FileField in the TemporaryUpload object
-        memfile = InMemoryUploadedFile(file_data, None, tuc.file_id,
-                                       'application/octet-stream',
-                                       tuc.total_size, None)
         tu = TemporaryUpload(upload_id=tuc.upload_id, file_id=tuc.file_id,
-                             file=memfile, upload_name=tuc.upload_name,
+                             file=chunked_file, upload_name=tuc.upload_name,
                              upload_type=TemporaryUpload.FILE_DATA,
                              uploaded_by=tuc.uploaded_by)
         tu.save()
 
         # Check that the final file is stored and of the correct size
+        chunk_dir = os.path.join(storage.base_location, tuc.upload_dir)
         stored_file_path = os.path.join(chunk_dir, tuc.file_id)
         if ((not os.path.exists(stored_file_path)) or
                 (not os.path.getsize(stored_file_path) == tuc.total_size)):
